#!/bin/env python

import gcore.EarthCoordTrans as eo
from gcore.StationDataBase import StationDataBase
import os, datetime, pandas, argparse
import matplotlib.pyplot as plt

class NMEAParser:
    def __init__(self, file_path, stadb_path='/GIPSY_source/goa-var/rtg/stationDatabase/sta_db_rt.itrf20', timeForStadb=763300800):
        self.file_path = file_path

        filename = os.path.basename(self.file_path)
        splitFileName = filename.split('_')
        self.year = int(splitFileName[1][0:4])
        self.doy = int(splitFileName[1][4:7])
        self.station = splitFileName[0][0:4]

        b = StationDataBase(stadb_path, timeForStadb, self.station)
        self.refXyz = b.posVel(self.station)[0]
        self.rot = eo.rotEnv2Xyz(self.refXyz).T

    def parse(self):
        data = []
        with open(self.file_path, 'r') as file:
            for line in file:
                line = line.strip()
                if line.startswith('$'):
                    parts = line.split(',')
                    sentence_type = parts[0][3:]
                    if sentence_type == 'GGA':
                        parsed_data = self.parse_GGA(parts)
                        if parsed_data:
                            data.append(parsed_data)
                    # Add more sentence types as needed
        return pandas.DataFrame(data)

    def parse_GGA(self, parts):
        if len(parts) < 15:
            return None
        try:
            time = self.parseTime(parts[1]) # UTC time

            latitude = parts[2]
            latitude_direction = parts[3]
            longitude = parts[4]
            longitude_direction = parts[5]
            fix_quality = int(parts[6])
            num_satellites = int(parts[7])
            horizontal_dilution = float(parts[8])
            altitude = float(parts[9])
            altitude_units = parts[10]
            geoidal_height = float(parts[11])
            geoidal_height_units = parts[12]
            age_of_differential = parts[13]
            differential_station_id = parts[14]
            
            latitude = self.parseLatLon(latitude, latitude_direction)
            longitude = self.parseLatLon(longitude, longitude_direction)

            if altitude_units != 'M':
                print('Found unusual unit for altitude ({:s}). Was expecting meters'.format(altitude_units))

            XYZ = eo.llh2xyz(latitude, longitude, altitude)

            ENU = self.rot.dot((XYZ - self.refXyz)).T

            parsed_data = {
                'time': time,
                'station':self.station,
                'ECEF': XYZ,
                'ENU': ENU,
                'fix_quality': fix_quality,
                'num_satellites': num_satellites,
                'horizontal_dilution': horizontal_dilution,
                'geoidal_height': geoidal_height,
                'geoidal_height_units': geoidal_height_units,
                'age_of_differential': age_of_differential,
                'differential_station_id': differential_station_id
            }
            return parsed_data
        except ValueError:
            return None
        
    def parseTime(self, t):
        # hhmmss.sss
        hour = int(t[0:2])
        minute = int(t[2:4])
        second = float(t[4:])

        time = datetime.datetime(self.year, 1, 1) + datetime.timedelta(days=self.doy-1, hours=hour, minutes=minute, milliseconds=second*1000)
        return time
    
    def parseLatLon(self, ll, direction): 
        # dddmm.mmmm
        dot_ = ll.find('.')
        deg = int(ll[:dot_-2])
        min = float(ll[dot_-2:])
        deg = deg + min / 60

        if direction in ('S', 'W'):
            return -deg
        return deg
    
def parseArguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("-mainFolder", required=True, type=str, default='/usr1/nnaciri/bnc/bnc_2.13/Output/pppSolutions', help="Path to the folder containing all stream-specific folders. Default is: /usr1/nnaciri/bnc/bnc_2.13/Output/pppSolutions")
    parser.add_argument("-stream", required=True, type=str, help="Name of the corrections stream - the folder name.")
    parser.add_argument("-station", required=True, type=str, help="Name of the station - use 9 char ID; JPLM00USA.")
    parser.add_argument("-year", required=False, default=2024, type=int, help="Year; defaults to 2024.")
    parser.add_argument("-doy", required=True, type=int, help="Day of year.")
    parser.add_argument("-outFolder", required=True, type=str, help="Output folder for the figure.")

    args = parser.parse_args()

    return (args.mainFolder, args.stream, args.station, args.year, args.doy, args.outFolder)
    
# Example usage:
if __name__ == "__main__":
    # stream_ = 'JPL42T'
    # station = 'JPLM00USA'
    # year = 2024
    # doy = 83
    # outFolder = '/home/nnaciri/code/pppSolutions_figures'

    mainFolder, stream_, station, year, doy, outFolder = parseArguments()

    if not os.path.isdir(outFolder):
        os.makedirs(outFolder)

    year = '{:04d}'.format(year)
    doy = '{:03d}'.format(doy)

    parser = NMEAParser(os.path.join(mainFolder, "{:s}/output_PPP/{:s}_{:s}{:s}0000_01D_01S.nmea".format(stream_, station, year, doy)))
    parsed_data = parser.parse()

    fig, ax = plt.subplots(figsize=(9, 6))
    ax.plot(parsed_data['time'], [x[0] for x in parsed_data['ENU']], 'o-', markersize=0.5, linewidth=0.1, label='E')
    ax.plot(parsed_data['time'], [x[1] for x in parsed_data['ENU']], 'o-', markersize=0.5, linewidth=0.1, label='N')
    ax.plot(parsed_data['time'], [x[2] for x in parsed_data['ENU']], 'o-', markersize=0.5, linewidth=0.1, label='U')

    t = min(parsed_data['time'])
    minTime = datetime.datetime(t.year, t.month, t.day, 0, 0, 0)
    maxTime = minTime + datetime.timedelta(hours=24)
    ax.set_xlim(minTime, maxTime)

    ax.grid(); ax.set_ylabel('Error [m]'); ax.set_ylim(-1, 1); ax.legend()
    fig.tight_layout()

    outFile = os.path.join(outFolder, '{:s}{:s}_{:s}_{:s}.png'.format(year, doy, stream_, station))
    fig.savefig(outFile, format='png', dpi=600)
